"use strict";
/**
 * Copyright(c) Microsoft Corporation.All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const botbuilder_dialogs_1 = require("botbuilder-dialogs");
const botframework_schema_1 = require("botframework-schema");
const i18next_1 = require("i18next");
const eventPrompt_1 = require("../dialogs/eventPrompt");
const extensions_1 = require("../extensions");
const remoteUserTokenProvider_1 = require("../remoteUserTokenProvider");
const responses_1 = require("../responses");
const tokenEvents_1 = require("../tokenEvents");
const authenticationResponses_1 = require("./authenticationResponses");
const providerTokenResponse_1 = require("./providerTokenResponse");
class MultiProviderAuthDialog extends botbuilder_dialogs_1.ComponentDialog {
    constructor(authenticationConnections, appCredentials) {
        super(MultiProviderAuthDialog.name);
        this.selectedAuthType = '';
        this.localAuthConfigured = false;
        this.authenticationConnections = authenticationConnections;
        this.appCredentials = appCredentials;
        this.responseManager = new responses_1.ResponseManager(['en', 'de', 'es', 'fr', 'it', 'zh'], [authenticationResponses_1.AuthenticationResponses]);
        const firstStep = [
            this.firstStep.bind(this)
        ];
        const remoteAuth = [
            this.sendRemoteEvent.bind(this),
            this.receiveRemoteEvent.bind(this)
        ];
        const localAuth = [
            this.promptForProvider.bind(this),
            this.promptForAuth.bind(this),
            this.handleTokenResponse.bind(this)
        ];
        this.addDialog(new botbuilder_dialogs_1.WaterfallDialog(DialogIds.firstStepPrompt, firstStep));
        // Add remote authentication support
        this.addDialog(new botbuilder_dialogs_1.WaterfallDialog(DialogIds.remoteAuthPrompt, remoteAuth));
        this.addDialog(new eventPrompt_1.EventPrompt(DialogIds.remoteAuthEventPrompt, tokenEvents_1.TokenEvents.tokenResponseEventName, this.tokenResponseValidator.bind(this)));
        // If authentication connections are provided locally then we enable "local auth"
        // otherwise we only enable remote auth where the calling Bot handles this for us.
        if (this.authenticationConnections !== undefined && this.authenticationConnections.length > 0) {
            let authDialogAdded = false;
            this.authenticationConnections.forEach((connection) => {
                // We ignore placeholder connections in config that don't have a Name
                if (connection.name !== '') {
                    const oauthPrompt = new botbuilder_dialogs_1.OAuthPrompt(connection.name, {
                        connectionName: connection.name,
                        title: i18next_1.default.t('common:login'),
                        text: i18next_1.default.t('common:loginDescription', connection.name)
                    }, this.authPromptValidator.bind(this));
                    this.addDialog(oauthPrompt);
                    authDialogAdded = true;
                }
            });
            // Only add Auth supporting local auth dialogs if we found valid authentication connections to use
            // otherwise it will just work in remote mode.
            if (authDialogAdded) {
                this.addDialog(new botbuilder_dialogs_1.WaterfallDialog(DialogIds.localAuthPrompt, localAuth));
                const prompt = new botbuilder_dialogs_1.ChoicePrompt(DialogIds.providerPrompt);
                prompt.style = botbuilder_dialogs_1.ListStyle.suggestedAction;
                this.addDialog(prompt);
                this.localAuthConfigured = true;
            }
            else {
                throw new Error('Something wrong with the authentication.');
            }
        }
    }
    async firstStep(stepContext) {
        if (remoteUserTokenProvider_1.isRemoteUserTokenProvider(stepContext.context.adapter)) {
            return stepContext.beginDialog(DialogIds.remoteAuthPrompt);
        }
        if (stepContext.context.activity.channelId === 'directlinespeech') {
            // Speech channel doesn't support OAuthPrompt./OAuthCards so we rely on tokens being set by the Linked Accounts technique
            // Therefore we don't use OAuthPrompt and instead attempt to directly retrieve the token from the store.
            if (stepContext.context.activity.from === undefined || stepContext.context.activity.from.id === '') {
                throw new Error('Missing From or From.Id which is required for token retrieval.');
            }
            if (this.appCredentials === undefined) {
                throw new Error('AppCredentials were not passed which are required for speech enabled authentication scenarios.');
            }
            // PENDING OAuthClient in botbuilder-connector // NOT IMPLEMENTED
            const connectionName = this.authenticationConnections[0].name;
            try {
                // Attempt to retrieve the token directly, we can't prompt the user for which Token to use so go with the first
                // Moving forward we expect to have a "default" choice as part of Linked Accounts.
                // PENDING get tokenResponse from OAuthClient
                throw new Error('Not implemented');
            }
            catch (error) {
                this.telemetryClient.trackEvent({
                    name: 'DirectLineSpeechTokenRetrievalFailure',
                    properties: {
                        // tslint:disable-next-line:no-unsafe-any
                        Exception: error.message
                    }
                });
            }
            const responseTokens = new Map([
                ['authType', connectionName]
            ]);
            const noLinkedAccountResponse = this.responseManager.getResponse(authenticationResponses_1.AuthenticationResponses.noLinkedAccount, responseTokens);
            await stepContext.context.sendActivity(noLinkedAccountResponse);
            // Enable Direct Line Speech clients to receive an event that will tell them
            // to trigger a sign-in flow when a token isn't present
            const requestOAuthFlowEvent = extensions_1.ActivityExtensions.createReply(stepContext.context.activity);
            requestOAuthFlowEvent.type = botframework_schema_1.ActivityTypes.Event;
            requestOAuthFlowEvent.name = 'RequestOAuthFlow';
            await stepContext.context.sendActivity(requestOAuthFlowEvent);
            return {
                status: botbuilder_dialogs_1.DialogTurnStatus.cancelled
            };
        }
        if (this.localAuthConfigured) {
            return stepContext.beginDialog(DialogIds.localAuthPrompt);
        }
        throw new Error('Local authentication is not configured, please check the auth connection section in your configuration file.');
    }
    async sendRemoteEvent(stepContext) {
        if (remoteUserTokenProvider_1.isRemoteUserTokenProvider(stepContext.context.adapter)) {
            // eslint-disable-next-line @typescript-eslint/tslint/config, @typescript-eslint/no-explicit-any
            const tokenProvider = stepContext.context.adapter;
            await tokenProvider.sendRemoteTokenRequestEvent(stepContext.context);
            // Wait for the tokens/response event
            return stepContext.prompt(DialogIds.remoteAuthEventPrompt, {});
        }
        throw new Error('The adapter does not support RemoteTokenRequest.');
    }
    async receiveRemoteEvent(stepContext) {
        if (stepContext.context.activity !== undefined && stepContext.context.activity.value !== undefined) {
            // eslint-disable-next-line @typescript-eslint/tslint/config
            const tokenResponse = JSON.parse(stepContext.context.activity.value);
            return stepContext.endDialog(tokenResponse);
        }
        throw new Error('Token Response is invalid.');
    }
    async promptForProvider(stepContext) {
        if (this.authenticationConnections.length === 1) {
            const result = this.authenticationConnections[0].name;
            return stepContext.next(result);
        }
        // DELTA inconsistences between IUserTokenProvider and BotFrameworkAdapter implementation
        const adapter = stepContext.context.adapter;
        if (adapter !== undefined) {
            const tokenStatusCollection = await adapter.getTokenStatus(stepContext.context, stepContext.context.activity.from.id);
            const matchingProviders = tokenStatusCollection.filter((p) => {
                return (p.hasToken || false) && this.authenticationConnections.some((t) => {
                    return t.name === p.connectionName;
                });
            });
            if (matchingProviders.length === 1) {
                const authType = matchingProviders[0].connectionName;
                return stepContext.next(authType);
            }
            if (matchingProviders.length > 1) {
                const choices = matchingProviders.map((connection) => {
                    const value = connection.connectionName || '';
                    return {
                        action: {
                            type: botframework_schema_1.ActionTypes.ImBack,
                            title: value,
                            value: value
                        },
                        value: value
                    };
                });
                return stepContext.prompt(DialogIds.providerPrompt, {
                    prompt: this.responseManager.getResponse(authenticationResponses_1.AuthenticationResponses.configuredAuthProvidersPrompt),
                    choices: choices
                });
            }
            else {
                const choices = this.authenticationConnections.map((connection) => {
                    return {
                        action: {
                            type: botframework_schema_1.ActionTypes.ImBack,
                            title: connection.name,
                            value: connection.name
                        },
                        value: connection.name
                    };
                });
                return stepContext.prompt(DialogIds.providerPrompt, {
                    prompt: this.responseManager.getResponse(authenticationResponses_1.AuthenticationResponses.authProvidersPrompt),
                    choices: choices
                });
            }
        }
        throw new Error('The adapter doesn\'t support Token Handling.');
    }
    async promptForAuth(stepContext) {
        if (typeof stepContext.result === 'string') {
            this.selectedAuthType = stepContext.result;
        }
        else {
            const choice = stepContext.result;
            if (choice !== undefined && choice.value !== undefined) {
                this.selectedAuthType = choice.value;
            }
        }
        return stepContext.prompt(this.selectedAuthType, {});
    }
    async handleTokenResponse(stepContext) {
        const tokenResponse = stepContext.result;
        if (tokenResponse !== undefined && tokenResponse.token) {
            const result = await this.createProviderTokenResponse(stepContext.context, tokenResponse);
            return stepContext.endDialog(result);
        }
        this.telemetryClient.trackEvent({
            name: 'TokenRetrievalFailure'
        });
        return { status: botbuilder_dialogs_1.DialogTurnStatus.cancelled };
    }
    async createProviderTokenResponse(context, tokenResponse) {
        const tokens = await this.getTokenStatus(context, context.activity.from.id);
        const match = tokens.find((t) => t.connectionName === tokenResponse.connectionName);
        if (!match) {
            throw new Error('Token not found');
        }
        const response = {
            authenticationProvider: providerTokenResponse_1.getAuthenticationProvider(match.serviceProviderDisplayName || ''),
            tokenResponse: tokenResponse
        };
        return Promise.resolve(response);
    }
    async getTokenStatus(context, userId, includeFilter) {
        if (context === undefined) {
            throw new Error('"context" undefined');
        }
        if (userId === undefined || userId === '') {
            throw new Error('"userId" undefined');
        }
        if (context.activity.channelId === 'directlinespeech') {
            if (this.appCredentials === undefined) {
                throw new Error('AppCredentials were not passed which are required for speech enabled authentication scenarios.');
            }
            // PENDING OAuthClient // DELTA
            return [];
        }
        else {
            const tokenProvider = context.adapter;
            if (tokenProvider !== undefined) {
                return tokenProvider.getTokenStatus(context, userId, includeFilter);
            }
            else {
                throw new Error('Adapter does not support IUserTokenProvider');
            }
        }
    }
    // Validators
    async tokenResponseValidator(promptContext) {
        const activity = promptContext.recognized.value;
        const result = activity !== undefined && activity.type === botframework_schema_1.ActivityTypes.Event;
        return Promise.resolve(result);
    }
    async authPromptValidator(promptContext) {
        const token = promptContext.recognized.value;
        if (token !== undefined && token.token !== '') {
            return Promise.resolve(true);
        }
        const eventActivity = promptContext.context.activity;
        if (eventActivity !== undefined && eventActivity.name === 'token/response') {
            promptContext.recognized.value = eventActivity.value;
            return Promise.resolve(true);
        }
        this.telemetryClient.trackEvent({
            name: 'AuthPromptValidatorAsyncFailure'
        });
        return Promise.resolve(false);
    }
}
exports.MultiProviderAuthDialog = MultiProviderAuthDialog;
var DialogIds;
(function (DialogIds) {
    DialogIds.providerPrompt = 'ProviderPrompt';
    DialogIds.firstStepPrompt = 'FirstStep';
    DialogIds.localAuthPrompt = 'LocalAuth';
    DialogIds.remoteAuthPrompt = 'RemoteAuth';
    DialogIds.remoteAuthEventPrompt = 'RemoteAuthEvent';
})(DialogIds || (DialogIds = {}));
//# sourceMappingURL=multiProviderAuthDialog.js.map