"use strict";
/**
 * Copyright(c) Microsoft Corporation.All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const botbuilder_dialogs_1 = require("botbuilder-dialogs");
const botframework_schema_1 = require("botframework-schema");
const extensions_1 = require("../extensions");
const interruptableDialog_1 = require("./interruptableDialog");
const interruptionAction_1 = require("./interruptionAction");
class RouterDialog extends interruptableDialog_1.InterruptableDialog {
    // Constructor
    constructor(dialogId, telemetryClient) {
        super(dialogId, telemetryClient);
    }
    async onBeginDialog(innerDc, options) {
        return this.onContinueDialog(innerDc);
    }
    async onContinueDialog(innerDc) {
        const status = await this.onInterruptDialog(innerDc);
        if (status === interruptionAction_1.InterruptionAction.MessageSentToUser) {
            // Resume the waiting dialog after interruption
            await innerDc.repromptDialog();
            return botbuilder_dialogs_1.Dialog.EndOfTurn;
        }
        else if (status === interruptionAction_1.InterruptionAction.StartedDialog) {
            // Stack is already waiting for a response, shelve inner stack
            return botbuilder_dialogs_1.Dialog.EndOfTurn;
        }
        else {
            const activity = innerDc.context.activity;
            if (extensions_1.ActivityExtensions.isStartActivity(activity)) {
                await this.onStart(innerDc);
            }
            switch (activity.type) {
                case botframework_schema_1.ActivityTypes.Message: {
                    // Note: This check is a workaround for adaptive card buttons that should map to an event
                    // (i.e. startOnboarding button in intro card)
                    if (activity.value) {
                        await this.onEvent(innerDc);
                    }
                    else if (activity.text !== undefined && activity.text !== '') {
                        const result = await innerDc.continueDialog();
                        switch (result.status) {
                            case botbuilder_dialogs_1.DialogTurnStatus.empty: {
                                await this.route(innerDc);
                                break;
                            }
                            case botbuilder_dialogs_1.DialogTurnStatus.complete: {
                                await this.complete(innerDc, result);
                                // End active dialog
                                await innerDc.endDialog();
                                break;
                            }
                            default:
                        }
                    }
                    break;
                }
                case botframework_schema_1.ActivityTypes.Event: {
                    await this.onEvent(innerDc);
                    break;
                }
                case botframework_schema_1.ActivityTypes.Invoke: {
                    // Used by Teams for Authentication scenarios.
                    await innerDc.continueDialog();
                    break;
                }
                default: {
                    await this.onSystemMessage(innerDc);
                }
            }
            return botbuilder_dialogs_1.Dialog.EndOfTurn;
        }
    }
    /**
     * Called when the inner dialog stack is complete.
     * @param innerDC - The dialog context for the component.
     * @param result - The dialog result when inner dialog completed.
     * @returns A Promise representing the asynchronous operation.
     */
    async complete(innerDc, result) {
        await innerDc.endDialog(result);
        return Promise.resolve();
    }
    /**
     * Called when an event activity is received.
     * @param innerDC - The dialog context for the component.
     * @returns A Promise representing the asynchronous operation.
     */
    async onEvent(innerDc) {
        return Promise.resolve();
    }
    /**
     * Called when a system activity is received.
     * @param innerDC - The dialog context for the component.
     * @returns A Promise representing the asynchronous operation.
     */
    async onSystemMessage(innerDc) {
        return Promise.resolve();
    }
    /**
     * Called when a conversation update activity is received.
     * @param innerDC - The dialog context for the component.
     * @returns A Promise representing the asynchronous operation.
     */
    async onStart(innerDc) {
        return Promise.resolve();
    }
    async onInterruptDialog(dc) {
        return Promise.resolve(interruptionAction_1.InterruptionAction.NoAction);
    }
}
exports.RouterDialog = RouterDialog;
//# sourceMappingURL=routerDialog.js.map