"use strict";
/**
 * Copyright(c) Microsoft Corporation.All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const azure_cognitiveservices_contentmoderator_1 = require("azure-cognitiveservices-contentmoderator");
const botframework_schema_1 = require("botframework-schema");
const ms_rest_azure_1 = require("ms-rest-azure");
const stream_1 = require("stream");
class ContentModeratorMiddleware {
    /**
     * Initializes a new instance of the "ContentModeratorMiddleware" class.
     * @param subscriptionKey Azure Service Key.
     * @param region Azure Service Region.
     */
    constructor(subscriptionKey, region) {
        /**
         * Key for Text Moderator result in Bot Context dictionary.
         */
        this.serviceName = 'ContentModerator';
        /**
         * Key for Text Moderator result in Bot Context dictionary.
         */
        this.textModeratorResultKey = 'TextModeratorResult';
        this.subscriptionKey = subscriptionKey;
        this.region = region;
    }
    /**
     * Analyzes activity text with Content Moderator and adds result to Bot Context. Run on each turn of the conversation.
     * @param context The Bot Context object.
     * @param next The next middleware component to run.
     */
    async onTurn(context, next) {
        if (context === undefined) {
            throw new Error('Context not found.');
        }
        if (context.activity.type === botframework_schema_1.ActivityTypes.Message) {
            const textStream = this.textToReadable(context.activity.text);
            const credentials = new ms_rest_azure_1.CognitiveServicesCredentials(this.subscriptionKey);
            const client = new azure_cognitiveservices_contentmoderator_1.ContentModeratorClient(credentials, `${this.region}.api.cognitive.microsoft.com`);
            const screenResult = await client.textModeration.screenText('text/plain', textStream, {
                language: 'eng',
                autocorrect: true,
                pII: true,
                listId: undefined,
                classify: true
            });
            context.turnState.set(this.textModeratorResultKey, screenResult);
        }
        await next();
    }
    textToReadable(text) {
        const readable = new stream_1.Readable();
        // tslint:disable-next-line:no-empty
        readable._read = () => { };
        readable.push(text);
        readable.push(undefined);
        return readable;
    }
}
exports.ContentModeratorMiddleware = ContentModeratorMiddleware;
//# sourceMappingURL=contentModeratorMiddleware.js.map