"use strict";
/**
 * Copyright(c) Microsoft Corporation.All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const botframework_schema_1 = require("botframework-schema");
const xml_js_1 = require("xml-js");
/**
 * Set Speech Synthesis Markup Language (SSML) on an Activity's Speak property with locale and voice input.
 */
class SetSpeakMiddleware {
    constructor(locale = DEFAULT_LOCALE, voiceName = DEFAULT_VOICE_FONT) {
        this.namespaceURI = 'https://www.w3.org/2001/10/synthesis';
        this.locale = locale;
        this.voiceFont = voiceName;
    }
    /**
     * If outgoing Activities are messages and using the Direct Line Speech channel,
     * decorate the Speak property with an SSML formatted string.
     * @param context The Bot Context object.
     * @param next The next middleware component to run.
     */
    async onTurn(context, next) {
        const handler = async (ctx, activities, nextSend) => {
            activities.forEach((activity) => {
                switch (activity.type) {
                    case botframework_schema_1.ActivityTypes.Message: {
                        if (activity.speak === undefined) {
                            activity.speak = activity.text;
                        }
                        // PENDING: Use Microsoft.Bot.Connector.Channels comparison when "directlinespeech" is available
                        if (activity.channelId === 'directlinespeech') {
                            activity.speak = this.decodeSSML(activity);
                        }
                        break;
                    }
                    default:
                }
            });
            return nextSend();
        };
        context.onSendActivities(handler);
        return next();
    }
    decodeSSML(activity) {
        if (activity.speak === undefined || activity.speak.trim() === '') {
            return '';
        }
        let rootElement = this.elementParse(activity.speak);
        if (rootElement === undefined || this.getLocalName(rootElement) !== 'speak') {
            // If the text is not valid XML, or if it's not a <speak> node, treat it as plain text.
            rootElement = this.createBaseElement(activity.speak);
        }
        this.addAttributeIfMissing(rootElement, 'version', '1.0');
        this.addAttributeIfMissing(rootElement, 'xml:lang', `lang${this.locale}`);
        this.addAttributeIfMissing(rootElement, 'xmlns:mstts', 'https://www.w3.org/2001/mstts');
        // Fix issue with 'number_digit' interpreter
        if (rootElement.elements !== undefined && rootElement.elements[0].elements !== undefined) {
            const sayAsElements = rootElement.elements[0].elements.filter((e) => e.name === 'say-as');
            sayAsElements.forEach((e) => {
                this.updateAttributeIfPresent(e, 'interpret-as', 'digits', 'number_digit');
            });
        }
        // add voice element if absent
        this.addVoiceElementIfMissing(rootElement, this.voiceFont);
        return xml_js_1.js2xml(rootElement, { compact: false });
    }
    elementParse(value) {
        try {
            return xml_js_1.xml2js(value, { compact: false });
        }
        catch (error) {
            return undefined;
        }
    }
    getLocalName(element) {
        if (element.elements !== undefined && element.elements.length === 1) {
            return element.elements[0].name;
        }
        return undefined;
    }
    createBaseElement(value) {
        // creating simple element
        const result = {
            elements: [
                {
                    type: 'element',
                    name: 'speak'
                }
            ]
        };
        // adding attributes
        result.attributes = {
            xmlns: this.namespaceURI
        };
        // adding sub-node
        result.elements = [
            {
                type: 'text',
                text: value
            }
        ];
        return result;
    }
    addAttributeIfMissing(element, attName, attValue) {
        if (element.elements !== undefined && element.elements[0] !== undefined) {
            if (element.elements[0].attributes === undefined) {
                element.elements[0].attributes = {
                    attName: attValue
                };
            }
            else {
                if (element.elements[0].attributes[attName] === undefined) {
                    element.elements[0].attributes[attName] = attValue;
                }
            }
        }
    }
    updateAttributeIfPresent(element, attName, attOld, attNew) {
        if (element.attributes !== undefined && element.attributes[attName] === attOld) {
            element.attributes[attName] = attNew;
        }
    }
    addVoiceElementIfMissing(element, attValue) {
        try {
            if (element.elements === undefined || element.elements[0].elements === undefined) {
                throw new Error('rootElement undefined');
            }
            const existingVoiceElement = element.elements[0].elements.find((e) => e.name === 'voice');
            // If an existing voice element is undefined (absent), then add it. Otherwise, assume the author has set it correctly.
            if (existingVoiceElement === undefined) {
                // tslint:disable-next-line:no-unsafe-any
                const oldElements = JSON.parse(JSON.stringify(element.elements[0].elements));
                element.elements[0].elements = [
                    {
                        type: 'element',
                        name: 'voice',
                        attributes: {
                            name: attValue
                        },
                        elements: oldElements
                    }
                ];
            }
            else {
                if (existingVoiceElement.attributes !== undefined) {
                    // tslint:disable-next-line:no-string-literal
                    existingVoiceElement.attributes['name'] = attValue;
                }
            }
        }
        catch (error) {
            // tslint:disable-next-line:no-unsafe-any
            throw new Error(`Could not add voice element to speak property: ${error.message}`);
        }
    }
}
exports.SetSpeakMiddleware = SetSpeakMiddleware;
const DEFAULT_LOCALE = 'en-US';
const DEFAULT_VOICE_FONT = 'Microsoft Server Speech Text to Speech Voice (en-US, JessaNeural)';
//# sourceMappingURL=setSpeakMiddleware.js.map