"use strict";
/**
 * Copyright(c) Microsoft Corporation.All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const botbuilder_1 = require("botbuilder");
const botbuilder_core_1 = require("botbuilder-core");
const botframework_schema_1 = require("botframework-schema");
const fs_1 = require("fs");
const i18next_1 = require("i18next");
const path_1 = require("path");
const card_1 = require("./card");
const responseTemplate_1 = require("./responseTemplate");
class ResponseManager {
    constructor(locales, responseTemplates) {
        this.defaultLocaleKey = 'default';
        this.jsonResponses = new Map();
        responseTemplates.forEach((responseTemplate) => {
            const resourceName = responseTemplate.name;
            const resource = responseTemplate.pathToResource || path_1.join(__dirname, '..', 'resources');
            this.loadResponses(resourceName, resource);
            locales.forEach((locale) => {
                try {
                    this.loadResponses(resourceName, resource, locale);
                }
                catch (_a) {
                    // If satellite assembly doesn't exist, bot will fall back to default.
                }
            });
        });
    }
    /**
     * Gets a simple response from template with Text, Speak, InputHint, and SuggestedActions set.
     * @param templateId The name of the response template.
     * @param tokens string map of tokens to replace in the response.
     * @returns An Activity.
     */
    getResponse(templateId, tokens) {
        const locale = i18next_1.default.language;
        const template = this.getResponseTemplate(templateId, locale);
        // create the response the data items
        return this.parseResponse(template, tokens);
    }
    /**
     * Get a response with an Adaptive Card attachment.
     * @param cards The card(s) to add to the response.
     * @returns An Activity.
     */
    getCardResponse(cards) {
        const locale = i18next_1.default.language;
        const resourcePath = path_1.join(__dirname, '..', 'resources', 'cards');
        if (cards instanceof card_1.Card) {
            const json = this.loadCardJson(cards.name, locale, resourcePath);
            const attachment = this.buildCardAttachment(json, cards.data);
            return botbuilder_core_1.MessageFactory.attachment(attachment);
        }
        else {
            const attachments = [];
            cards.forEach((card) => {
                const json = this.loadCardJson(card.name, locale, resourcePath);
                attachments.push(this.buildCardAttachment(json, card.data));
            });
            return botbuilder_core_1.MessageFactory.carousel(attachments);
        }
    }
    /**
     * Get a response from template with Text, Speak, InputHint, SuggestedActions, and an Adaptive Card attachment.
     * @param templateId The name of the response template.
     * @param cards The card(s) object to add to the response.
     * @param tokens Optional string map of tokens to replace in the response.
     */
    getCardResponseWithTemplateId(templateId, cards, tokens) {
        const response = this.getResponse(templateId, tokens);
        const locale = i18next_1.default.language;
        const resourcePath = path_1.join(__dirname, '..', 'resources', 'cards');
        if (cards instanceof card_1.Card) {
            const json = this.loadCardJson(cards.name, locale, resourcePath);
            const attachment = this.buildCardAttachment(json, cards.data);
            return botbuilder_core_1.MessageFactory.attachment(attachment, response.text, response.speak, response.inputHint);
        }
        else {
            const attachments = [];
            cards.forEach((card) => {
                const json = this.loadCardJson(card.name, locale, resourcePath);
                attachments.push(this.buildCardAttachment(json, card.data));
            });
            return botbuilder_core_1.MessageFactory.carousel(attachments, response.text, response.speak, response.inputHint);
        }
    }
    /**
     * Get a response from template with Text, Speak, InputHint, SuggestedActions, and a Card attachments with list items inside.
     * @param templateId The name of the response template.
     * @param card The main card container contains list.
     * @param tokens Optional string map of tokens to replace in the response.
     * @param containerName Target container.
     * @param containerItems Card list which will be injected to target container.
     * @returns An Activity.
     */
    getCardResponseWithContainer(templateId, card, tokens, containerName, containerItems) {
        const locale = i18next_1.default.language;
        const resourcePath = path_1.join(__dirname, '..', 'resources', 'cards');
        const json = this.loadCardJson(card.name, locale, resourcePath);
        const emailOverviewCard = this.buildCard(json, card.data);
        if (containerName && emailOverviewCard.body) {
            const itemContainer = emailOverviewCard.body.find((item) => {
                return item.type === 'Container' && item.id === containerName;
            });
            if (itemContainer && containerItems) {
                containerItems.forEach((cardItem) => {
                    const itemJson = this.loadCardJson(cardItem.name, locale, resourcePath);
                    const itemCard = this.buildCard(itemJson, cardItem.data);
                    if (itemCard.body && itemCard.body[0]) {
                        // eslint-disable-next-line @typescript-eslint/tslint/config
                        const items = itemCard.body[0].items;
                        items.push(itemContainer);
                    }
                });
            }
        }
        const attachment = botbuilder_core_1.CardFactory.adaptiveCard(emailOverviewCard);
        if (templateId) {
            const response = this.getResponse(templateId, tokens);
            return botbuilder_core_1.MessageFactory.attachment(attachment, response.text, response.speak, response.inputHint);
        }
        return botbuilder_core_1.MessageFactory.attachment(attachment);
    }
    getResponseTemplate(templateId, locale) {
        let localeKey = locale !== undefined ? locale : i18next_1.default.language;
        // warm up the JsonResponses loading to see if it actually exist.
        // If not, throw with the loading time exception that's actually helpful
        let key = this.getJsonResponseKeyForLocale(templateId, localeKey);
        // if no matching json file found for locale, try parent language
        if (key === undefined) {
            localeKey = localeKey.split('-')[0]
                .toLowerCase();
            key = this.getJsonResponseKeyForLocale(templateId, localeKey);
            // fall back to default
            if (key === undefined) {
                localeKey = this.defaultLocaleKey;
                key = this.getJsonResponseKeyForLocale(templateId, localeKey);
            }
        }
        if (key === undefined) {
            throw new Error();
        }
        // Get the bot response from the .json file
        const responseLocale = this.jsonResponses.get(localeKey);
        if (!responseLocale || !responseLocale.has(key)) {
            throw new Error(`Unable to find response ${templateId}`);
        }
        const response = responseLocale.get(key);
        if (response === undefined) {
            throw new Error();
        }
        return response;
    }
    format(messageTemplate, tokens) {
        let result = messageTemplate;
        ResponseManager.complexTokensRegex.lastIndex = 0;
        let match = ResponseManager.complexTokensRegex.exec(result);
        while (match) {
            const bindingJson = match[0];
            const tokenKey = bindingJson
                .replace('{', '')
                .replace('}', '');
            if (tokens && tokens.has(tokenKey)) {
                result = result.replace(bindingJson, tokens.get(tokenKey) || '');
            }
            match = ResponseManager.complexTokensRegex.exec(result);
        }
        return result;
    }
    loadResponses(resourceName, resourcePath, locale) {
        // if locale is not set, add resources under the default key.
        const localeKey = (locale !== undefined) ? locale : this.defaultLocaleKey;
        let jsonPath = path_1.join(resourcePath, `${resourceName}.${localeKey}.json`);
        try {
            require.resolve(jsonPath);
        }
        catch (errLocale) {
            jsonPath = path_1.join(resourcePath, `${resourceName}.json`);
            // Search for the common resource
            try {
                require.resolve(jsonPath);
            }
            catch (err) {
                throw new Error(`Unable to find '${resourceName}' in '${resourcePath}'`);
            }
        }
        try {
            // eslint-disable-next-line @typescript-eslint/tslint/config
            const content = JSON.parse(this.jsonFromFile(jsonPath));
            const localeResponses = this.jsonResponses.get(localeKey) || new Map();
            Object.entries(content)
                .forEach((val) => {
                const key = val[0];
                const value = val[1];
                const template = Object.assign(new responseTemplate_1.ResponseTemplate(), value);
                localeResponses.set(key, template);
            });
            this.jsonResponses.set(localeKey, localeResponses);
        }
        catch (err) {
            throw new Error(`Error deserializing ${jsonPath}`);
        }
    }
    getJsonResponseKeyForLocale(responseId, locale) {
        if (this.jsonResponses.has(locale)) {
            const localeResponses = this.jsonResponses.get(locale);
            if (localeResponses) {
                return localeResponses.has(responseId) ? responseId : undefined;
            }
        }
        return undefined;
    }
    parseResponse(template, data) {
        const reply = template.reply;
        if (!reply) {
            throw new Error('There is no reply in the ResponseTemplate');
        }
        if (reply.text) {
            reply.text = this.format(reply.text, data);
        }
        if (reply.speak) {
            reply.speak = this.format(reply.speak, data);
        }
        const activity = {
            type: botbuilder_1.ActivityTypes.Message,
            text: reply.text,
            speak: reply.speak,
            inputHint: template.inputHint
        };
        if (template.suggestedActions !== undefined && template.suggestedActions.length > 0) {
            activity.suggestedActions = {
                actions: [],
                to: []
            };
            template.suggestedActions.forEach((action) => {
                if (activity.suggestedActions) {
                    activity.suggestedActions.actions.push({
                        type: botframework_schema_1.ActionTypes.ImBack,
                        title: action,
                        value: action
                    });
                }
            });
        }
        activity.attachments = [];
        return activity;
    }
    loadCardJson(cardName, locale, resourcePath) {
        let jsonFile = path_1.join(resourcePath, `${cardName}.${locale}.json`);
        try {
            require.resolve(jsonFile);
        }
        catch (errLocale) {
            try {
                jsonFile = path_1.join(resourcePath, `${cardName}.json`);
                require.resolve(jsonFile);
            }
            catch (error) {
                throw new Error(`Could not file Adaptive Card resource ${jsonFile}`);
            }
        }
        return this.jsonFromFile(jsonFile);
    }
    buildCardAttachment(json, data) {
        const card = this.buildCard(json, data);
        return botbuilder_core_1.CardFactory.adaptiveCard(card);
    }
    buildCard(json, data) {
        let jsonOut = json;
        // If cardData was provided
        if (data !== undefined) {
            // add all properties to the list
            const tokens = new Map();
            // get property names from cardData
            Object.entries(data)
                .forEach((entry) => {
                if (!tokens.has(entry[0])) {
                    tokens.set(entry[0], entry[1]);
                }
            });
            // replace tokens in json
            ResponseManager.simpleTokensRegex.lastIndex = 0;
            let match = ResponseManager.simpleTokensRegex.exec(jsonOut);
            while (match) {
                if (tokens.has(match[0])) {
                    jsonOut = jsonOut.replace(match[0], tokens.get(match[0]) || '');
                }
                match = ResponseManager.simpleTokensRegex.exec(jsonOut);
            }
        }
        // Deserialize/Serialize logic is needed to prevent JSON exception in prompts
        // eslint-disable-next-line @typescript-eslint/tslint/config
        return JSON.parse(jsonOut);
    }
    jsonFromFile(filePath) {
        // tslint:disable-next-line:non-literal-fs-path
        return fs_1.readFileSync(filePath, 'utf8');
    }
}
ResponseManager.simpleTokensRegex = /\{(\w+)\}/g;
ResponseManager.complexTokensRegex = /\{[^{\}]+(?=})\}/g;
exports.ResponseManager = ResponseManager;
//# sourceMappingURL=responseManager.js.map