"use strict";
/**
 * Copyright(c) Microsoft Corporation.All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const botbuilder_1 = require("botbuilder");
const timers_1 = require("timers");
const extensions_1 = require("../extensions");
const util_1 = require("../util");
class InProcAdapter extends botbuilder_1.BotAdapter {
    constructor() {
        super();
        this.lastId = 0;
        this.queuedActivities = [];
    }
    get nextId() {
        return (this.lastId + 1).toString();
    }
    async processActivity(activity, callback, messageReceivedHandler) {
        this.messageReceivedHandler = messageReceivedHandler;
        const context = new botbuilder_1.TurnContext(this, activity);
        await this.runMiddleware(context, callback);
    }
    use(middleware) {
        super.use(middleware);
        return this;
    }
    async continueConversation(reference, logic) {
        if (reference === undefined) {
            throw new Error('Missing parameter.  reference is required');
        }
        if (logic === undefined) {
            throw new Error('Missing parameter.  logic is required');
        }
        const context = new botbuilder_1.TurnContext(this, extensions_1.ActivityExtensions.getContinuationActivity(reference));
        await this.runMiddleware(context, logic);
    }
    getNextReply() {
        return this.queuedActivities.pop();
    }
    getReplies() {
        return this.queuedActivities
            .splice(0, this.queuedActivities.length)
            .reverse();
    }
    async sendActivities(context, activities) {
        const responses = [];
        const proactiveActivities = [];
        activities.forEach(async (activity) => {
            if (!activity.id) {
                activity.id = this.nextId;
            }
            if (!activity.timestamp) {
                activity.timestamp = new Date();
            }
            if (activity.type === 'delay') {
                // The BotFrameworkAdapter and Console adapter implement this
                // hack directly in the POST method. Replicating that here
                // to keep the behavior as close as possible to facilitate
                // more realistic tests.
                // eslint-disable-next-line @typescript-eslint/tslint/config
                const delayMs = activity.value;
                await this.sleep(delayMs);
            }
            else {
                if (activity.deliveryMode === util_1.CommonUtil.deliveryModeProactive) {
                    proactiveActivities.push(activity);
                }
                else {
                    this.queuedActivities.push(activity);
                }
            }
            responses.push({ id: activity.id });
        });
        if (proactiveActivities.length > 0 && this.backgroundTaskQueue !== undefined && this.messageReceivedHandler !== undefined) {
            this.backgroundTaskQueue.queueBackgroundWorkItem(async () => {
                if (this.messageReceivedHandler !== undefined) {
                    await this.messageReceivedHandler(proactiveActivities);
                }
            });
        }
        return responses;
    }
    async sleep(delay) {
        return new Promise((resolve) => {
            timers_1.setTimeout(resolve, delay);
        });
    }
    async updateActivity(context, activity) {
        throw new Error('Method not used.');
    }
    async deleteActivity(context, reference) {
        throw new Error('Method not used.');
    }
}
exports.InProcAdapter = InProcAdapter;
//# sourceMappingURL=inProcAdapter.js.map